/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 * Project  Zero
 *
 * (C) Copyright IBM Corp. 2007  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
  */
package com.ibm.p8.engine.xapi.groovy;

import com.ibm.phpj.reflection.XAPIPassSemantics;
import com.ibm.phpj.xapi.ExtensionBaseImpl;
import com.ibm.phpj.xapi.RuntimeContext;
import com.ibm.phpj.xapi.annotations.XAPIArguments;
import com.ibm.phpj.xapi.annotations.XAPIExtension;
import com.ibm.phpj.xapi.annotations.XAPIFunction;

/**
 * Documents the PHP/Groovy bridge extension functions.
 */
@XAPIExtension("groovy")
public final class GroovyExtensionLibrary extends ExtensionBaseImpl {

    /**
     * Imports a Groovy class defined in a Groovy script into the PHP runtime.
     * <p>
     * The Groovy script is located by searching the include path of the PHP runtime.
     * <p>
     * Groovy classes can be imported in to the PHP runtime using the <code>groovy_import</code>
     * extension function. This creates a PHP class which has the same shape as the original
     * Groovy class, including methods and fields (both static and instance). Any interactions
     * with the imported Groovy class are redirected automatically to the underlying Groovy class.
     * <p>
     * <code>groovy_import</code> imports both Groovy classes and interfaces.
     * <p>
     * When a Groovy class is imported, the resulting PHP class can optionally be defined to
     * implement one or more PHP interfaces. This is a convenience for PHP scripts as many
     * Groovy classes implement interfaces that are not required in PHP.
     * <p>
     * <code>groovy_import</code> will optionally declare a super class for the class being
     * imported. The super class must already have been imported into the PHP runtime.
     * <p>
     * To avoid class name collisions it is sometimes useful to rename classes during the
     * import. An optional class name can be passed to <code>groovy_import</code>.
     *
     * @param string $script_name the file name of the Groovy script containing a class
     *
     * @param array $interfaces an optional array of PHP interfaces names for the
     *  imported Groovy class to declare as implemented
     *
     * @param boolean $include_base_class <code>true</code> for the PHP class to
     *  declare the same base class as the Groovy class being imported
     *
     * @param string $php_class_name the name of the class when it is created in
     *  the PHP runtime - this is optional and so can be <code>NULL</code>
     *  in which case the PHP class name is set the same as the Groovy class name
     *
     * @return boolean <code>true</code> if the import succeeded; <code>false</code> otherwise
     */
    @XAPIFunction("groovy_import")
    public void groovy_import(RuntimeContext runtimeContext) { }

    /**
     * Executes a snippet of Groovy script and returns the result.
     * @param string $code the Groovy code to be evaluated
     * @return object the return value from evaluating the Groovy script
     */
    @XAPIFunction("groovy_eval")
    public void groovy_eval(RuntimeContext runtimeContext) { }

    /**
    * Creates a Groovy compatible closure around an object or function.
    * @param callback $callback any value PHP callback such as an array containing an object
    *  and the name of a method, or the name of a function.
    * @return object a Groovy compatible closure
    */
    @XAPIFunction("groovy_create_closure")
    @XAPIArguments(DefaultPassSemantics = XAPIPassSemantics.ByValue)
    public void groovy_create_closure(RuntimeContext runtimeContext) { }
}

