/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 * Project  Zero
 *
 * (C) Copyright IBM Corp. 2007  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
 * Copyright (c) 1999 - 2006 The PHP Group. All rights reserved.
 * ============================================================================
 */
package com.ibm.p8.engine.xapi.reflection.impl;

import com.ibm.phpj.reflection.XAPIPassSemantics;
import com.ibm.phpj.xapi.ExtensionBaseImpl;
import com.ibm.phpj.xapi.RuntimeContext;
import com.ibm.phpj.xapi.annotations.XAPIArguments;
import com.ibm.phpj.xapi.annotations.XAPIExtension;
import com.ibm.phpj.xapi.annotations.XAPIFunction;

/**
 * Documents the PHP/Java bridge extension functions.
 */
@XAPIExtension("java")
public final class JavaExtensionLibrary extends ExtensionBaseImpl {

    /**
     * Imports a Java class definition into the PHP runtime.
     * <p>
     * Java classes can be imported in to the PHP runtime using the <code>java_import</code>
     * extension function. This creates a PHP class which has the same shape as the original
     * Java class, including methods and fields (both static and instance). Any interactions
     * with the imported Java class are redirected automatically to the underlying Java class.
     * <p>
     * <code>java_import</code> imports both Java classes and interfaces.
     * <p>
     * When a Java class is imported, the resulting PHP class can optionally be defined to
     * implement one or more PHP interfaces. This is a convenience for PHP scripts as many
     * Java classes implement interfaces that are not required in PHP.
     * <p>
     * <code>java_import</code> will optionally declare a super class for the class being
     * imported. The super class must already have been imported into the PHP runtime.
     * <p>
     * To avoid class name collisions it is sometimes useful to rename classes during the
     * import. An optional class name can be passed to <code>java_import</code>.
     * <p>
     * PHP does not have the concept of an inner class. By default all static inner classes
     * defined in a Java class that is imported will also be imported as top level PHP classes.
     * Only static inner classes can be imported by the Java bridge. When a PHP class is
     * created by importing a Java static inner class, the resulting PHP class is given the
     * name of the inner class without using the name of the enclosing Java class.
     * <p>
     * For example: If a Java Class named <code>Zoo</code> which has a static inner class
     * named <code>Zoo.Tiger</code>, is imported then this will result in a PHP class named
     * <code>Zoo</code> and a PHP class named <code>Tiger</code>.
     * <p>
     * Importing static inner classes as top level classes is convenient but has the potential
     * to cause naming conflicts. Such a conflict can be resolved by choosing to import the
     * outer class without importing inner classes. Any required static inner classes can then
     * be explicitly imported. If required the inner classes can be renamed on the import.
     *
     * @param string $java_class_name the fully qualified Java class name to import
     *
     * @param array $interfaces an optional array of PHP interfaces names for the
     *  imported class to declare as implemented
     *
     * @param boolean $include_base_class <code>true</code> for the PHP class to
     *  declare the same base class as the Java class being imported
     *
     * @param string $php_class_name the name of the class when it is created in
     *  the PHP runtime - this is optional and so can be <code>NULL</code>
     *  in which case the PHP class name is set the same as the Java class name
     *
     * @param boolean $include_nested_classes <code>true</code> to automatically
     *  import any nested inner classes defined in the Java class
     *
     * @return boolean <code>true</code> if the import succeeded; <code>false</code> otherwise
     */
    @XAPIFunction("java_import")
    public void java_import(RuntimeContext runtimeContext) { }
}

